import logging
from google.oauth2.credentials import Credentials
import google.auth.transport.requests

class GoogleOAuth2Service:
    def __init__(self):
        self.logger = logging.getLogger(__name__)

    def auth(self, file):
        self.logger.info("Logging in")
        credentials = Credentials.from_authorized_user_file(file)
        request = google.auth.transport.requests.Request()
        credentials.refresh(request)
        self.logger.info("Logged in")
        return credentials



import pytest
from unittest.mock import MagicMock, patch
from happtiq_commons_google_cloud.gcs_api_service import GcsApiService

@pytest.fixture
def gcs_api_service(monkeypatch):
    monkeypatch.setattr("google.auth.default", MagicMock(return_value=(MagicMock(universe_domain="googleapis.com"),"some-proje")))
    return GcsApiService(timeout=30)

def mock_storage_client():
    mock_blob = MagicMock()
    mock_blob.download_as_text.return_value = "Downloaded text"
    mock_bucket = MagicMock()
    mock_bucket.blob.return_value = mock_blob
    mock_client = MagicMock()
    mock_client.bucket.return_value = mock_bucket    
    return mock_client

def test_download_as_text(gcs_api_service, monkeypatch):
    monkeypatch.setattr(gcs_api_service, "client", mock_storage_client())

    bucket_name = "test-bucket"
    source_blob_name = "test-file.txt"
    
    text = gcs_api_service.download_as_text(bucket_name, source_blob_name)
    assert text == "Downloaded text"
    gcs_api_service.client.bucket.assert_called_once_with(bucket_name)
    gcs_api_service.client.bucket().blob.assert_called_once_with(source_blob_name)

def test_upload(gcs_api_service, monkeypatch):
    mock_client = MagicMock()
    mock_client.bucket.return_value = MagicMock(blob=MagicMock(upload_from_string=MagicMock()))
    monkeypatch.setattr(gcs_api_service, "client", mock_client)

    fileContent = b"Test file content"
    bucket_name = "test-bucket"
    destination = "test-file.txt"
    content_type = "text/plain"

    result = gcs_api_service.upload(fileContent, bucket_name, destination, content_type)
    assert result == f"gs://{bucket_name}/{destination}"
    gcs_api_service.client.bucket.assert_called_once_with(bucket_name)
    gcs_api_service.client.bucket().blob.assert_called_once_with(destination)
    gcs_api_service.client.bucket().blob(destination).upload_from_string.assert_called_once_with(fileContent, content_type=content_type, timeout=30)

def test_upload_file(gcs_api_service, monkeypatch):
    mock_client = MagicMock()
    mock_client.bucket.return_value = MagicMock(blob=MagicMock(upload_from_filename=MagicMock()))
    monkeypatch.setattr(gcs_api_service, "client", mock_client)

    file_path = "test-file.txt"
    bucket_name = "test-bucket"
    destination = "test-file.txt"
    content_type = "text/plain"

    result = gcs_api_service.upload_file(file_path, bucket_name, destination, content_type)
    assert result == f"gs://{bucket_name}/{destination}"
    gcs_api_service.client.bucket.assert_called_once_with(bucket_name)
    gcs_api_service.client.bucket().blob.assert_called_once_with(destination)
    gcs_api_service.client.bucket().blob(destination).upload_from_filename.assert_called_once_with(file_path, content_type=content_type, timeout=30)

def test_upload_file_gzipped(gcs_api_service, monkeypatch):
    mock_client = MagicMock()
    mock_client.bucket.return_value = MagicMock(blob=MagicMock(upload_from_filename=MagicMock()))
    monkeypatch.setattr(gcs_api_service, "client", mock_client)

    file_path = "test-file.txt"
    bucket_name = "test-bucket"
    destination = "test-file.txt"
    content_type = "text/plain"

    result = gcs_api_service.upload_file(file_path, bucket_name, destination, content_type, gzipped=True)
    assert result == f"gs://{bucket_name}/{destination}"
    gcs_api_service.client.bucket.assert_called_once_with(bucket_name)
    gcs_api_service.client.bucket().blob.assert_called_once_with(destination)
    gcs_api_service.client.bucket().blob(destination).upload_from_filename.assert_called_once_with(file_path, content_type=content_type, timeout=30)
    gcs_api_service.client.bucket().blob(destination).content_encoding = "gzip"

def test_list_files_with_prefix(gcs_api_service, monkeypatch):
    mock_client = MagicMock()
    mock_bucket = MagicMock()
    mock_blob1 = MagicMock()
    mock_blob1.name = "file1.txt"
    mock_blob2 = MagicMock()
    mock_blob2.name = "file2.txt"
    mock_blobs = [mock_blob1,mock_blob2]

    mock_client.bucket.return_value = mock_bucket
    mock_bucket.list_blobs.return_value = mock_blobs
    monkeypatch.setattr(gcs_api_service, "client", mock_client)

    bucket_name = "test-bucket"
    prefix = "test-prefix/"

    result = gcs_api_service.list_files_with_prefix(bucket_name, prefix)

    assert result == ["file1.txt", "file2.txt"]
    mock_client.bucket.assert_called_once_with(bucket_name)
    mock_bucket.list_blobs.assert_called_once_with(prefix=prefix)



import pytest
from happtiq_commons_google_cloud.auth_service import GoogleOAuth2Service
from unittest.mock import MagicMock, patch
from google.oauth2.credentials import Credentials

@pytest.fixture
def auth_service():
    return GoogleOAuth2Service()

def test_auth(auth_service, monkeypatch):
    mock_credentials = MagicMock(spec=Credentials)
    mock_credentials.refresh.return_value = None
    monkeypatch.setattr(Credentials, "from_authorized_user_file", MagicMock(return_value=mock_credentials))

    file = "test_credentials.json"
    credentials = auth_service.auth(file)
    assert credentials == mock_credentials
    Credentials.from_authorized_user_file.assert_called_once_with(file)
    mock_credentials.refresh.assert_called_once()

def test_auth_error(auth_service, monkeypatch):
    mock_credentials = MagicMock(spec=Credentials)
    mock_credentials.refresh.side_effect = Exception("Error refreshing credentials")
    monkeypatch.setattr(Credentials, "from_authorized_user_file", MagicMock(return_value=mock_credentials))

    file = "test_credentials.json"
    with pytest.raises(Exception):
        auth_service.auth(file)



import pytest
from happtiq_commons_google_cloud.pubsub_api_service import PubsubApiService
from unittest.mock import MagicMock
import json


@pytest.fixture
def pubsub_api_service(monkeypatch):
    monkeypatch.setattr("google.auth.default", MagicMock(return_value=(MagicMock(),"some-proje")))
    return PubsubApiService()

def test_publish_message(pubsub_api_service, monkeypatch):
    mock_publisher = MagicMock()
    monkeypatch.setattr(pubsub_api_service, "publisher", mock_publisher)

    topic = "projects/test-project/topics/test-topic"
    message = {"key": "value"}
    pubsub_api_service.publish_message(topic, message)
    mock_publisher.publish.assert_called_once_with(topic, json.dumps(message).encode('utf-8'))

def test_build_topic_path(pubsub_api_service):
    project_id = "test-project"
    topic_id = "test-topic"
    topic_path = pubsub_api_service.build_topic_path(project_id, topic_id)
    assert topic_path == "projects/test-project/topics/test-topic"

def test_publish_message_error(pubsub_api_service, monkeypatch):
    mock_publisher = MagicMock()
    mock_publisher.publish.side_effect = Exception("Error publishing message")
    monkeypatch.setattr(pubsub_api_service, "publisher", mock_publisher)
    topic = "projects/test-project/topics/test-topic"
    message = {"key": "value"}
    with pytest.raises(Exception):
        pubsub_api_service.publish_message(topic, message)



from google.cloud import secretmanager
import logging


class SecretManagerApiService:
    def __init__(self):
        self.client = secretmanager.SecretManagerServiceClient()
        self.logger = logging.getLogger(__name__)

    def read_secret(self, secret_name: str):
        self.logger.info(f"Getting secret {secret_name}")
        
        try:
            response = self.client.access_secret_version(request={"name": secret_name})
            self.logger.info(f"Got secret {secret_name} successfully")
        except Exception as e:
            self.logger.error(f"Failed to acess secret {secret_name}: {e}")
            raise

        # Extract 'Plain Text' Key
        try:
            self.logger.debug(f"Decoding secret {secret_name}")
            secret_key = response.payload.data.decode("UTF-8")
            self.logger.info(f"Decoded secret {secret_name} successfully")
        except Exception as e:
            self.logger.error(f"Failed to decode secret: {e}")
            raise

        return secret_key
    
    def build_secret_name(project_id: str, secret_id: str, version_id: str = "latest"):
        return f"projects/{project_id}/secrets/{secret_id}/versions/{version_id}"



from google.cloud import storage
import logging

class GcsApiService:
    def __init__(self, timeout):
        self.client = storage.Client()
        self.logger = logging.getLogger(__name__)
        self.timeout = timeout


    def download_as_text(self, bucket_name: str, source_blob_name: str) -> str:
        self.logger.info(f"Downloading {bucket_name}/{source_blob_name}")
        bucket = self.client.bucket(bucket_name)
        blob = bucket.blob(source_blob_name)
        
        try:
            text = blob.download_as_text()
            self.logger.info(f"Downloaded {len(text)} chars from {bucket_name}/{source_blob_name}")
        except Exception as e:
            self.logger.error(f"Failed to download file {bucket_name}/{source_blob_name}: {e}")
            raise e
        
        return text
    
    def upload(self, fileContent: bytes,  bucket_name: str, destination: str, content_type: str) -> str:
        bucket = self.client.bucket(bucket_name)
        blob = bucket.blob(destination)
        blob.upload_from_string(fileContent, content_type=content_type, timeout=self.timeout)
        self.logger.info(f"File uploaded to {destination}.")
        return f"gs://{bucket_name}/{destination}"
    
    def upload_file(self, file_path: bytes,  bucket_name: str, destination: str, content_type: str, gzipped = False) -> str:
        bucket = self.client.bucket(bucket_name)
        blob = bucket.blob(destination)
        if gzipped:
            blob.content_encoding = "gzip"
        blob.upload_from_filename(file_path, content_type=content_type, timeout=self.timeout)
        self.logger.info(f"File uploaded to {destination}.")
        return f"gs://{bucket_name}/{destination}"
    
    def list_files_with_prefix(self, bucket_name: str, prefix: str):
        self.logger.info(f"Listing files in bucket {bucket_name} with prefix {prefix}")
        bucket = self.client.bucket(bucket_name)
        blobs = bucket.list_blobs(prefix=prefix)

        file_list = [blob.name for blob in blobs]
        self.logger.debug(f"Files in bucket {bucket_name} with prefix {prefix}: {file_list}")
        self.logger.info(f"found {len(file_list)} files in bucket {bucket_name} with prefix {prefix}")
    
        return file_list



import json
import logging
from google.cloud import pubsub_v1


class PubsubApiService:
    def __init__(self):
        self.publisher = pubsub_v1.PublisherClient()
        self.logger = logging.getLogger(__name__)

    def publish_message(self, topic, message: dict):
        try:
            self.logger.info(f"Publishing message {message} to topic {topic}")

            data = json.dumps(message).encode('utf-8')
            future = self.publisher.publish(topic, data)
            future.result()
            
            self.logger.info(f"Published message to pubsub {message}")
        except Exception as e:
            self.logger.error(f"Error publishing {message}: {e}")
            raise

    def build_topic_path(self, project_id: str, topic_id: str):
        return self.publisher.topic_path(project_id, topic_id)



import pytest
from happtiq_commons_google_cloud.input_parser_service import InputParserService

@pytest.fixture
def input_parser_service():
    return InputParserService()

def test_parse_input_with_json(input_parser_service):
    data = {
        "message": {
            "data": "eyJ0ZXN0IjogeyJrZXkiOiAidmFsdWUiLCAiYW5vdGhlciI6ICJpbnB1dCJ9LCAidGVzdDIiOiAic29tZXRoaW5nIn0="
        }
    }
    
    message_data = input_parser_service.parse_input(data)
    assert message_data == {"test": {"key": "value", "another": "input"}, "test2": "something"}

def test_parse_input_error(input_parser_service):
    data = {
        "message": {
            "data": "invalid-base64"
        }
    }
    with pytest.raises(Exception):
        input_parser_service.parse_input(data)



import pytest
from happtiq_commons_google_cloud.secret_manager_api_service import SecretManagerApiService
from unittest.mock import MagicMock, patch

@pytest.fixture
def secret_manager_api_service(monkeypatch):
    monkeypatch.setattr("google.auth.default", MagicMock(return_value=(MagicMock(),"some-proje")))
    return SecretManagerApiService()

def test_read_secret(secret_manager_api_service, monkeypatch):
    secret_value = "test-secret"
    
    mock_client = MagicMock()
    mock_client.access_secret_version.return_value = MagicMock(payload=MagicMock(data=bytes(secret_value, "utf-8")))
    monkeypatch.setattr(secret_manager_api_service, "client", mock_client)
    
    secret_name = "some-secret"
    secret_key = secret_manager_api_service.read_secret(secret_name)
    assert secret_key == secret_value

def test_build_secret_name():
    project_id = "test-project"
    secret_id = "test-secret"
    secret_name = SecretManagerApiService.build_secret_name(project_id, secret_id)
    assert secret_name == "projects/test-project/secrets/test-secret/versions/latest"

def test_read_secret_error(secret_manager_api_service, monkeypatch):
    mock_client = MagicMock()
    mock_client.access_secret_version.side_effect = Exception("Error accessing secret")
    monkeypatch.setattr(secret_manager_api_service, "client", mock_client)
    
    secret_name = "some-secret"
    with pytest.raises(Exception):
        secret_manager_api_service.read_secret(secret_name)



import pytest
from happtiq_commons_google_cloud.logging_utils import is_cloud_function, setup_logging
from unittest.mock import MagicMock, patch
import os

@pytest.fixture
def empty_fixture():
    return

def test_is_cloud_function_with_k_service(monkeypatch):
    monkeypatch.setenv('K_SERVICE', 'test-service')
    assert is_cloud_function()

def test_is_cloud_function_without_k_service(monkeypatch):
    monkeypatch.delenv('K_SERVICE', raising=False)
    assert not is_cloud_function()



import logging
import json
import base64

class InputParserService:
    def __init__(self):
        self.logger = logging.getLogger(__name__)


    def parse_input(self, data):
        self.logger.info(f"Decoding data {data}")
        base64message = data["message"]["data"]
        pubsub_message = base64.b64decode(base64message).decode('utf-8')
        self.logger.debug(f"Decoded message {pubsub_message}")
        
        self.logger.debug(f"Parsing message {pubsub_message} as json")
        message_data = json.loads(pubsub_message)
        self.logger.info(f"Parsed message {message_data}")

        return message_data



import google.cloud.logging
import logging
import os

def setup_logging():
    if is_cloud_function():
        setup_cloud_function_logger()
    else:
        logging.basicConfig(level=logging.INFO)

def is_cloud_function():
    K_SERVICE = os.getenv('K_SERVICE')
    return K_SERVICE != None

def setup_cloud_function_logger():
    google.cloud.logging.Client().setup_logging()
